//
// Created by Sergei Shubin <s.v.shubin@gmail.com>
//

#ifndef BACKGROUND_TEXTURE_DECORATOR_H
#define BACKGROUND_TEXTURE_DECORATOR_H

#include "ShellOpenGL.h"
#include <Rocket/Core/Decorator.h>

class BackgroundTextureDecorator: public Rocket::Core::Decorator {
public:
	virtual ~BackgroundTextureDecorator();

    bool Initialise(const Rocket::Core::String& image_source, const Rocket::Core::String& image_path, GLint wrap_s, GLint wrap_t);

	/// Called on a decorator to generate any required per-element data for a newly decorated element.
	/// @param element[in] The newly decorated element.
	/// @return A handle to a decorator-defined data handle, or NULL if none is needed for the element.
	virtual Rocket::Core::DecoratorDataHandle GenerateElementData(Rocket::Core::Element *element);
	/// Called to release element data generated by this decorator.
	/// @param element_data[in] The element data handle to release.
	virtual void ReleaseElementData(Rocket::Core::DecoratorDataHandle element_data);

	/// Called to render the decorator on an element.
	/// @param element[in] The element to render the decorator on.
	/// @param element_data[in] The handle to the data generated by the decorator for the element.
	virtual void RenderElement(Rocket::Core::Element *element, Rocket::Core::DecoratorDataHandle element_data);
private:
	int m_texture;
    GLuint m_texture_id;
    GLsizei m_texture_width;
    GLsizei m_texture_height;
    GLint m_wrap_s, m_wrap_t;
};

#endif
